
using System;

using Microsoft.Cci;
using Microsoft.Tools.FxCop.Sdk;
using Microsoft.Tools.FxCop.Sdk.Introspection;


namespace FxCop.Rules
{
    /// <summary>
    ///        <para>
    ///            Provides the base <see langword="abstract"/> class for all introspection rules.
    ///        </para>
    /// </summary>
    [CLSCompliant(false)]
    public abstract class BaseRule : BaseIntrospectionRule
    {
        /// <summary>
        ///        <para>
        ///            Initializes a new instance of the <see cref="BaseRule"/> class with the specified name.
        ///        </para>
        /// </summary>
        /// <param name="name">
        ///        A <see cref="String"/> containing the name of the rule.
        /// </param>
        protected BaseRule(string name)
            : base(name, "FxCop.Rules.Rules", typeof(BaseRule).Assembly)
        {
        }

        /// <summary>
        ///        <para>
        ///            This member overrides <see cref="BaseIntrospectionRule.Check(TypeNode)"/>.
        ///        </para>
        /// </summary>
        /// <param name="type">
        ///        The <see cref="TypeNode"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="type"/>.
        /// </returns>
        public override ProblemCollection Check(TypeNode type)
        {
            if (type == null)
                return null;

            switch (type.NodeType)
            {
                case NodeType.Class:
                    {
                        return Check((Class)type);
                    }
                case NodeType.Interface:
                    {
                        return Check((Interface)type);
                    }
                case NodeType.Struct:
                    {
                        return Check((Struct)type);
                    }
                case NodeType.EnumNode:
                    {
                        return Check((EnumNode)type);
                    }
            }

            return base.Check(type);

        }

        /// <summary>
        ///        <para>
        ///            This member overrides <see cref="BaseIntrospectionRule.Check(Module)"/>.
        ///        </para>
        /// </summary>
        /// <param name="module">
        ///        The <see cref="Module"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="module"/>.
        /// </returns>
        public override ProblemCollection Check(Module module)
        {
            if (module == null)
                return null;

            if (module.NodeType == NodeType.Assembly)
            {
                return Check((AssemblyNode)module);
            }

            return base.Check(module);
        }

        /// <summary>
        ///        <para>
        ///            Checks the specified <see cref="AssemblyNode"/>.
        ///        </para>
        /// </summary>
        /// <param name="a">
        ///        The <see cref="AssemblyNode"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="a"/>.
        /// </returns>
        public virtual ProblemCollection Check(AssemblyNode a)
        {
            return null;
        }

        /// <summary>
        ///        <para>
        ///            Checks the specified <see cref="EnumNode"/>.
        ///        </para>
        /// </summary>
        /// <param name="e">
        ///        The <see cref="EnumNode"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="e"/>.
        /// </returns>
        public virtual ProblemCollection Check(EnumNode e)
        {
            return null;
        }

        /// <summary>
        ///        <para>
        ///            Checks the specified <see cref="Class"/>.
        ///        </para>
        /// </summary>
        /// <param name="c">
        ///        The <see cref="Class"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="c"/>.
        /// </returns>
        public virtual ProblemCollection Check(Class c)
        {
            return null;
        }

        /// <summary>
        ///        <para>
        ///            Checks the specified <see cref="Interface"/>.
        ///        </para>
        /// </summary>
        /// <param name="i">
        ///        The <see cref="Class"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="i"/>.
        /// </returns>
        public virtual ProblemCollection Check(Interface i)
        {
            return null;
        }

        /// <summary>
        ///        <para>
        ///            Checks the specified <see cref="Struct"/>.
        ///        </para>
        /// </summary>
        /// <param name="s">
        ///        The <see cref="Struct"/> to check.
        /// </param>
        /// <returns>
        ///        A <see cref="ProblemCollection"/> containing the problems associated with <paramref name="s"/>.
        /// </returns>
        public virtual ProblemCollection Check(Struct s)
        {
            return null;
        }
    }
}

